package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.ArticleEntity
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.domain.contract.FeedsContract
import com.biganto.visual.roompark.domain.custom_exception.CustomApiException
import com.biganto.visual.roompark.domain.model.ArticlePreviewModel
import com.biganto.visual.roompark.domain.model.ArticlesPreviewModel
import com.biganto.visual.roompark.domain.model.FeedModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import timber.log.Timber.e
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

//@Singleton
class FeedsContractModule @Inject constructor(
    private val api: IRoomParkApi,
    private val db: IDb
): FeedsContract {



    override fun fetchFeeds(): Observable<List<FeedModel>> =
        fetchAllFeeds().doOnError (::e)


    override fun fetchFeedObservable(id: Int): Observable<ArticlesPreviewModel> =
        fetchArticles(id)

    override fun getArticle(id: Int): Observable<ArticlePreviewModel> {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    init {
        Timber.d("Feeds Repository Created")
    }


    private fun fetchArticlessApi(feedId:Int): Observable<List<ArticleEntity>>? {
        val feed = db.getFeed(feedId)?.firstOrNull()
        if (feed == null) {
            e("Unknown feedId: $feedId -  cann't resolve feed_alias!")
            throw CustomApiException.UnknownCustomApiException(-1,null,"Unknown Feed!")
        }
        else {
            return api.getArticlesPage(feed.alias, 11, 1)
                .doOnNext { Timber.d("raw0 $it") }
                .map { it.items }
                .map { fromRaw(it, feedId, feed.alias) }
                .doOnNext(db::blockingUpsert)
                .subscribeOn(Schedulers.io())
        }
    }


    private fun fetchArticlesDb(feedId:Int): Observable<MutableList<ArticleEntity>>? {
        val feed = db.getFeed(feedId)?.firstOrNull()
        if (feed == null) {
            e("Unknown feedId: $feedId -  cann't resolve feed_alias!")
            throw CustomApiException.UnknownCustomApiException(-1, null, "Unknown Feed!")
        } else return db.fetchArticles(feed.alias, 11, 1)
            .toList()
            .toObservable()
            .subscribeOn(Schedulers.io())
    }

    private fun fetchArticles(feedId:Int): Observable<ArticlesPreviewModel> =
        Observable.mergeDelayError(
            arrayListOf(fetchArticlessApi(feedId),fetchArticlesDb(feedId))
        ).map { fromEntity(feedId,it) }


    private fun fetchFeedsApi() =
        api.getFeeds()
            .doOnNext { Timber.d("raw0 $it") }
            .map(::fromRaw)
            .doOnNext(db::blockingUpsert)
            .subscribeOn(Schedulers.io())

    private val fetchFeedsDb =
        db.fetchFeeds()
            .toList()
            .toObservable()
            .subscribeOn(Schedulers.io())

    fun fetchAllFeeds(): Observable<List<FeedModel>> =
        Observable.mergeDelayError(
            arrayListOf(fetchFeedsApi(),fetchFeedsDb)
        ).map { fromEntity(it,::fromEntity) }

}

